@@ -9,7 +9,7 @@ from torch.utils.data import DataLoader
 from pytorch_lightning.callbacks import ModelCheckpoint
 
 from neuralnets.data.datasets import LabeledVolumeDataset
-from neuralnets.networks.unet import UNet2D
+from neuralnets.networks.unet import UNet2D, UNet3D
 from neuralnets.util.losses import CrossEntropyLoss
 
 
@@ -176,3 +176,54 @@ class UNet2DClassifier(PLClassifier):
 
         return self
 
+
+class UNet3DClassifier(PLClassifier):
+
+    def __init__(self, epochs=10, gpus=(0,), accelerator='dp', log_dir='logs', log_freq=50, log_refresh_rate=None,
+                 train_batch_size=1, test_batch_size=1, num_workers=1, device=0, orientations=(0,),
+                 normalization='unit', transform=None, input_shape=(1, 256, 256), in_channels=1, coi=(0, 1),
+                 feature_maps=64, levels=4, skip_connections=True, residual_connections=False, norm='instance',
+                 activation='relu', dropout=0.0, loss_fn=CrossEntropyLoss(), lr=1e-3):
+        super().__init__(epochs=epochs, gpus=gpus, accelerator=accelerator, log_dir=log_dir, log_freq=log_freq,
+                         log_refresh_rate=log_refresh_rate, train_batch_size=train_batch_size,
+                         test_batch_size=test_batch_size, num_workers=num_workers, device=device,
+                         orientations=orientations, normalization=normalization, transform=transform)
+
+        # parameters
+        self.input_shape = input_shape
+        self.in_channels = in_channels
+        self.feature_maps = feature_maps
+        self.levels = levels
+        self.skip_connections = skip_connections
+        self.residual_connections = residual_connections
+        self.dropout = dropout
+        self.norm = norm
+        self.activation = activation
+        self.coi = coi
+        self.loss_fn = loss_fn
+        self.lr = lr
+
+
+    def fit(self, X, y):
+
+        # initialize model and trainer
+        self.model = UNet3D(input_shape=self.input_shape, in_channels=self.in_channels, feature_maps=self.feature_maps,
+                            levels=self.levels, skip_connections=self.skip_connections,
+                            residual_connections=self.residual_connections, dropout_enc=self.dropout,
+                            dropout_dec=self.dropout, norm=self.norm, activation=self.activation, coi=self.coi,
+                            loss_fn=self.loss_fn, lr=self.lr)
+        self.trainer = pl.Trainer(max_epochs=self.epochs, gpus=self.gpus, accelerator=self.accelerator,
+                                  default_root_dir=self.log_dir, flush_logs_every_n_steps=self.log_freq,
+                                  log_every_n_steps=self.log_freq, callbacks=self.callbacks,
+                                  progress_bar_refresh_rate=self.log_refresh_rate)
+
+        # construct dataloader
+        train = LabeledVolumeDataset(X, y, input_shape=self.model.input_shape, batch_size=self.train_batch_size,
+                                     transform=self.transform)
+        loader = DataLoader(train, batch_size=self.train_batch_size, num_workers=self.num_workers, pin_memory=True)
+
+        # train the network
+        self.trainer.fit(self.model, loader)
+
+        return self
+
